@@ -26,6 +26,7 @@ import iso8601
 import msgpack
 from oslo_config import cfg
 from oslo_log import log
+from oslo_serialization import msgpackutils
 from oslo_utils import timeutils
 import pandas
 import six
@@ -65,24 +66,14 @@ class CarbonaraBasedStorage(storage.StorageDriver):
 
     def __init__(self, conf):
         super(CarbonaraBasedStorage, self).__init__(conf)
-        self.coord = coordination.get_coordinator(
-            conf.coordination_url,
-            str(uuid.uuid4()).encode('ascii'))
         self.aggregation_workers_number = conf.aggregation_workers_number
         if self.aggregation_workers_number == 1:
             # NOTE(jd) Avoid using futures at all if we don't want any threads.
             self._map_in_thread = self._map_no_thread
         else:
             self._map_in_thread = self._map_in_futures_threads
-        self.start()
-
-    @utils.retry
-    def start(self):
-        try:
-            self.coord.start(start_heart=True)
-        except Exception as e:
-            LOG.error("Unable to start coordinator: %s" % e)
-            raise utils.Retry(e)
+        self.coord, my_id = utils.get_coordinator_and_start(
+            conf.coordination_url)
 
     def stop(self):
         self.coord.stop()
@@ -372,10 +363,20 @@ class CarbonaraBasedStorage(storage.StorageDriver):
     _MEASURE_SERIAL_FORMAT = "Qd"
     _MEASURE_SERIAL_LEN = struct.calcsize(_MEASURE_SERIAL_FORMAT)
 
-    def _unserialize_measures(self, data):
+    def _unserialize_measures(self, measure_id, data):
         nb_measures = len(data) // self._MEASURE_SERIAL_LEN
-        measures = struct.unpack(
-            "<" + self._MEASURE_SERIAL_FORMAT * nb_measures, data)
+        try:
+            measures = struct.unpack(
+                "<" + self._MEASURE_SERIAL_FORMAT * nb_measures, data)
+        except struct.error:
+            # This either a corruption, either a v2 measures
+            try:
+                return msgpackutils.loads(data)
+            except ValueError:
+                LOG.error(
+                    "Unable to decode measure %s, possible data corruption",
+                    measure_id)
+                raise
         return six.moves.zip(
             pandas.to_datetime(measures[::2], unit='ns'),
             itertools.islice(measures, 1, len(measures), 2))
@@ -453,13 +454,13 @@ class CarbonaraBasedStorage(storage.StorageDriver):
     def upgrade(self, index):
         marker = None
         while True:
-            metrics = index.list_metrics(limit=self.UPGRADE_BATCH_SIZE,
-                                         marker=marker)
-            for m in metrics:
-                self._check_for_metric_upgrade(m)
+            metrics = [(metric,) for metric in
+                       index.list_metrics(limit=self.UPGRADE_BATCH_SIZE,
+                                          marker=marker)]
+            self._map_in_thread(self._check_for_metric_upgrade, metrics)
             if len(metrics) == 0:
                 break
-            marker = metrics[-1].id
+            marker = metrics[-1][0].id
 
     def process_new_measures(self, indexer, metrics_to_process, sync=False):
         metrics = indexer.list_metrics(ids=metrics_to_process)
